/*
 * @file ads127l11.c
 *
 * @brief ADS127L11 Descriptor
 *
 * @copyright Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "ads127l11.h"


//****************************************************************************
//
// Internal variables
//
//****************************************************************************

// Array used to recall device register map configurations */
static uint8_t             registerMap[NUM_REGISTERS];


//****************************************************************************
//
// Internal function prototypes
//
//****************************************************************************

uint8_t     buildSPIarray(const uint8_t opcodeArray[], uint8_t byteArray[]);
uint8_t     enforce_selected_device_modes(uint8_t data);
uint8_t     getWordByteLength(void);



//*****************************************************************************
//
//! Getter function to access registerMap array from outside of this module.
//!
//! \fn uint8_t getRegisterValue(uint8_t address)
//!
//! NOTE: The internal registerMap arrays stores the last know register value,
//! since the last read or write operation to that register. This function
//! does not communicate with the device to retrieve the current register value.
//! For the most up-to-date register data or retrieving the value of a hardware
//! controlled register, it is recommend to use readSingleRegister() to read the
//! current register value.
//!
//! \return unsigned 8-bit register value.
//
//*****************************************************************************
uint16_t getRegisterValue(uint8_t address)
{
    assert(address < NUM_REGISTERS);
    return registerMap[address];
}



//*****************************************************************************
//
//! Example start up sequence for the ADS127L11.
//!
//! \fn void adcStartup(void)
//!
//! Before calling this function, the device must be powered,
//! the SPI/GPIO pins of the MCU must have already been configured,
//! and (if applicable) the external clock source should be provided to CLKIN.
//!
//! \return None.
//
//*****************************************************************************
void adcStartup(void)
{
	/* (OPTIONAL) Provide additional delay time for power supply settling */
	delay_ms(50);

	/* (OPTIONAL) Populate CRC lookup table */
	initCRC();

	/* (REQUIRED) Set nRESET pin high for ADC operation */
	setRESET(HIGH);

    /* (REQUIRED) Set START pin high to begin conversions    */
    setSTART(HIGH);

	/* (OPTIONAL) Toggle nRESET pin to ensure default register settings. */
	toggleRESET();

    /* (REQUIRED) Initialize internal 'registerMap' array with device default settings */
	restoreRegisterDefaults();

    /* (REQUIRED) Configure CONFIG4 register settings
     * NOTE: This function call is required here for this particular code implementation to work.
     * This function will enforce the CONFIG4 register settings as selected in the 'ads127l11.h' header file.
     *
     * Add static flag to allow all register writes for mode changes. Then remove after startup
     */
    writeSingleRegister(CONFIG4_REG_ADDRESS, CONFIG4_REG_DEFAULT);

	/* (OPTIONAL) Define your initial register settings here */
	// Example: Enable input buffers and VCM
    writeSingleRegister(CONFIG1_REG_ADDRESS, (CONFIG1_REG_DEFAULT | ((uint8_t) 0x13)));

}



//*****************************************************************************
//
//! Reads the contents of a single register at the specified address.
//!
//! \fn uint8_t readSingleRegister(uint8_t address)
//!
//! \param address is the 8-bit address of the register to read.
//!
//! \return Returns the 8-bit register read result.
//
//*****************************************************************************
uint8_t readSingleRegister(uint8_t address)
{
	/* Check that the register address is in range */
	assert(address < NUM_REGISTERS);

	// Build TX and RX byte array --NOTE maybe move the data arrays outside function to save memory
    uint8_t dataTx[SPI_BUFFER_SIZE] = { 0 };
    uint8_t dataRx[SPI_BUFFER_SIZE] = { 0 };

    // Build opcode and SPI TX array
    const uint8_t       arbitrary = 0x00;
    uint8_t             opcode[2] = {(OPCODE_RREG | address), arbitrary};
    uint8_t         numberOfBytes = buildSPIarray(opcode, dataTx);

	// [FRAME 1] Send RREG command
	spiSendReceiveArrays(dataTx, dataRx, numberOfBytes);

	// [FRAME 2] Send NULL command to retrieve the register data
	uint8_t nullopcode[2] = {OPCODE_NULL, OPCODE_NULL};
	registerMap[address] = sendCommand(nullopcode);

	return registerMap[address];
}



//*****************************************************************************
//
//! Writes data to a single register.
//!
//! \fn void writeSingleRegister(uint8_t address, uint8_t data)
//!
//! \param address is the address of the register to write to.
//! \param data is the value to write.
//!
//!
//! \return None.
//
//*****************************************************************************
void writeSingleRegister(uint8_t address, uint8_t data)
{
    /* Check that the register address is in range */
    assert(address < NUM_REGISTERS);

    // (OPTIONAL) Check if write operation will result in a reset
    // and restore defaults to ensure register map stays in sync
    if (address == CONTROL_REG_ADDRESS & data == CONTROL_REG_RESET_COMMAND)
        restoreRegisterDefaults();

    // (OPTIONAL) Enforce certain register field values when
    // writing to the CONFIG4 register to fix the operation mode
    if (CONFIG4_REG_ADDRESS == address)
    {
        data = enforce_selected_device_modes(data);
    }

    // Build TX and RX byte array
    uint8_t dataTx[SPI_BUFFER_SIZE] = { 0 };
    uint8_t dataRx[SPI_BUFFER_SIZE] = { 0 };

    // Build opcode and SPI TX array
    uint8_t   opcode[2]     = {(OPCODE_WREG | address), data};
    uint8_t numberOfBytes   = buildSPIarray(opcode, dataTx);

    // Send command
    spiSendReceiveArrays(dataTx, dataRx, numberOfBytes);

    // Update internal array
    registerMap[address] = data;

    // (RECOMMENDED) Read back register to confirm register write was successful
    readSingleRegister(address);

    // NOTE: Enabling the CRC words in the SPI command will NOT prevent an invalid write
}



//*****************************************************************************
//
//! Reads ADC data.
//!
//! \fn void readData(int32_t     adcdata)
//!
//! NOTE: Should be called after /DRDY goes low, and not during a /DRDY falling edge!
//!
//!
//!
//
//*****************************************************************************
int32_t readData(adc_channel_data *dataStruct)
{
    // Build TX and RX byte array
    uint8_t dataTx[SPI_BUFFER_SIZE] = { 0 };
    uint8_t dataRx[SPI_BUFFER_SIZE] = { 0 };

    // Build opcode and SPI TX array
    uint8_t opcode[2]     = {OPCODE_NULL, OPCODE_NULL};
    uint8_t numberOfBytes = buildSPIarray(opcode, dataTx);

    // Send command
    spiSendReceiveArrays(dataTx, dataRx, numberOfBytes);

    // Parse ADC response...
    uint8_t byteIndex = STATUS_ENABLED ? 1 : 0;
    int32_t signedExtendedData = signExtend(&dataRx[byteIndex]);
    if (dataStruct) //Check if dataStruct is a null pointer
    {
        if (STATUS_ENABLED)
        {
        dataStruct->status = dataRx[0];
        }

        if (SPI_CRC_ENABLED)
        {
            byteIndex = byteIndex + (RES_24_BIT_ENABLED ? 3 : 2);
            dataStruct->crc = dataRx[byteIndex];
        }

        dataStruct->adcdata = signedExtendedData;
    }   

    // Return ADC code
    return signedExtendedData;
}



//*****************************************************************************
//
//! Sends the specified SPI command to the ADC (NULL, STANDBY, or WAKEUP).
//!
//! \fn uint8_t sendCommand(uint8_t opcode[])
//!
//! \param opcode SPI command byte.
//!
//! NOTE: Other commands have their own dedicated functions to support
//! additional functionality.
//!
//! \return ADC response byte (typically the STATUS byte).
//
//*****************************************************************************
uint8_t sendCommand(uint8_t opcode[])
{
    /* Assert if this function is used to send any of the following opcodes */
    assert(OPCODE_RREG != opcode[0]);      /* Use "readSingleRegister()"   */
    assert(OPCODE_WREG != opcode[0]);      /* Use "writeSingleRegister()"  */


    // Build TX and RX byte array
    uint8_t dataTx[SPI_BUFFER_SIZE] = { 0 };
    uint8_t dataRx[SPI_BUFFER_SIZE] = { 0 };

    uint8_t numberOfBytes = buildSPIarray(opcode, dataTx);

    /* Set the nCS pin LOW */
    setCS(LOW);

    // Send the opcode
    int i;
    for (i = 0; i < numberOfBytes; i++)
    {
       dataRx[i] = spiSendReceiveByte(dataTx[i]);
    }

    /* Set the nCS pin HIGH */
    setCS(HIGH);

    // Return response byte
    uint8_t adcResponse = dataRx[(STATUS_ENABLED ? 1 : 0)];
    return adcResponse;
}



//*****************************************************************************
//
//! Resets the device by SPI command to CONTROL register
//!
//! \fn void resetDeviceByCommand(void)
//!
//! NOTE: This function does not capture DOUT data, but it could be modified
//! to do so.
//!
//! \return None.
//
//*****************************************************************************
void resetDeviceByCommand(void)
{
    //Reset by SPI Register Write

    // Write data 0x58 to CONTROL register
    uint8_t address    =   CONTROL_REG_ADDRESS;
    uint8_t data       =   CONTROL_REG_RESET_COMMAND;

    // Build TX and RX byte array
    uint8_t dataTx[SPI_BUFFER_SIZE] = { 0 };
    uint8_t dataRx[SPI_BUFFER_SIZE] = { 0 };

    // Build opcode and SPI TX array
    uint8_t     opcode[2] = {(OPCODE_WREG | address), data};
    uint8_t numberOfBytes = buildSPIarray(opcode, dataTx);

    // Send command
    spiSendReceiveArrays(dataTx, dataRx, numberOfBytes);

    // tSRLRST delay
    delay_ms(1);

    // Update register setting array to keep software in sync with device
    restoreRegisterDefaults();
}

//*****************************************************************************
//
//! Resets the device by SPI input pattern
//!
//! \fn void resetDeviceByPattern(void)
//!
//! NOTE: This function does not capture DOUT data, but it could be modified
//! to do so.
//!
//! \return None.
//
//*****************************************************************************
void resetDeviceByPattern(void)
{

#ifdef SPI_3_WIRE
    //Reset by SPI Input Pattern Option 1 (3 wire or 4 wire SPI mode)

    uint8_t junk;

    /* Send 1024 ones followed by zero*/
    int i;
    for (i = 0; i < 128; i++)
        {
           junk = spiSendReceiveByte((uint8_t) 0xFF);
        }
    junk = spiSendReceiveByte((uint8_t) 0x00);

#else

    //Reset by SPI Input Pattern Option 2 (recommended for daisy-chain operation -- 4 wire SPI mode only)

    /* Set the nCS pin LOW */
    setCS(LOW);

    int i;
    for (i = 0; i < 128; i++)
        {
           uint8_t junk = spiSendReceiveByte((uint8_t) 0xFF);
        }

    /* Set the nCS pin HIGH */
    setCS(HIGH);

#endif

    // tSRLRST delay
    delay_ms(1);

    // Update register setting array to keep software in sync with device
    restoreRegisterDefaults();
}



//*****************************************************************************
//
//! Updates the registerMap[] array to its default values.
//!
//! \fn void restoreRegisterDefaults(void)
//!
//! NOTES:
//! - If the MCU keeps a copy of the ADS127L11 register settings in memory,
//! then it is important to ensure that these values remain in sync with the
//! actual hardware settings. In order to help facilitate this, this function
//! should be called after powering up or resetting the device (either by
//! hardware pin control or SPI software command).
//!
//! - Reading back all of the registers after resetting the device can
//! accomplish the same result; however, this might be problematic if the
//! device was previously in CRC mode or STATUS output mode, since
//! resetting the device exits these modes. If the MCU is not aware of this
//! mode change, then read register commands will return invalid data due to
//! the expectation of data appearing in a different byte position.
//!
//! \return None.
//
//*****************************************************************************
void restoreRegisterDefaults(void)
{


    registerMap[DEVID_REG_ADDRESS]          =   DEVID_REG_DEFAULT;
//  registerMap[REVID_REG_ADDRESS]          =   REVID_REG_DEFAULT;
    registerMap[STATUS_REG_ADDRESS]         =   STATUS_REG_DEFAULT;
    registerMap[CONTROL_REG_ADDRESS]        =   CONTROL_REG_DEFAULT;
    registerMap[MUX_REG_ADDRESS]            =   MUX_REG_DEFAULT;
    registerMap[CONFIG1_REG_ADDRESS]        =   CONFIG1_REG_DEFAULT;
    registerMap[CONFIG2_REG_ADDRESS]        =   CONFIG2_REG_DEFAULT;
    registerMap[CONFIG3_REG_ADDRESS]        =   CONFIG3_REG_DEFAULT;
    registerMap[CONFIG4_REG_ADDRESS]        =   CONFIG4_REG_DEFAULT;
    registerMap[OFFSET_MSB_REG_ADDRESS]     =   OFFSET_MSB_REG_DEFAULT;
    registerMap[OFFSET_MID_REG_ADDRESS]     =   OFFSET_MID_REG_DEFAULT;
    registerMap[OFFSET_LSB_REG_ADDRESS]     =   OFFSET_LSB_REG_DEFAULT;
    registerMap[GAIN_MSB_REG_ADDRESS]       =   GAIN_MSB_REG_DEFAULT;
    registerMap[GAIN_MID_REG_ADDRESS]       =   GAIN_MID_REG_DEFAULT;
    registerMap[GAIN_LSB_REG_ADDRESS]       =   GAIN_LSB_REG_DEFAULT;
//  registerMap[CRC_REG_ADDRESS]            =   CRC_REG_DEFAULT;
}



//****************************************************************************
//
// Helper functions
//
//****************************************************************************


//*****************************************************************************
//
//! Takes a 16-bit word and returns the most-significant byte.
//!
//! \fn uint8_t upperByte(uint16_t uint16_Word)
//!
//! \param temp_word is the original 16-bit word.
//!
//! \return 8-bit most-significant byte.
//
//*****************************************************************************
uint8_t upperByte(uint16_t uint16_Word)
{
    uint8_t msByte;
    msByte = (uint8_t) ((uint16_Word >> 8) & 0x00FF);

    return msByte;
}



//*****************************************************************************
//
//! Takes a 16-bit word and returns the least-significant byte.
//!
//! \fn uint8_t lowerByte(uint16_t uint16_Word)
//!
//! \param temp_word is the original 16-bit word.
//!
//! \return 8-bit least-significant byte.
//
//*****************************************************************************
uint8_t lowerByte(uint16_t uint16_Word)
{
    uint8_t lsByte;
    lsByte = (uint8_t) (uint16_Word & 0x00FF);

    return lsByte;
}



//*****************************************************************************
//
//! Takes two 8-bit words and returns a concatenated 16-bit word.
//!
//! \fn uint16_t combineBytes(uint8_t upperByte, uint8_t lowerByte)
//!
//! \param upperByte is the 8-bit value that will become the MSB of the 16-bit word.
//! \param lowerByte is the 8-bit value that will become the LSB of the 16-bit word.
//!
//! \return concatenated 16-bit word.
//
//*****************************************************************************
uint16_t combineBytes(uint8_t upperByte, uint8_t lowerByte)
{
    uint16_t combinedValue;
    combinedValue = ((uint16_t) upperByte << 8) | ((uint16_t) lowerByte);

    return combinedValue;
}



//*****************************************************************************
//
//! Combines ADC data bytes into a single signed 32-bit word.
//!
//! \fn int32_t combineDataBytes(const uint8_t dataBytes[])
//!
//! \param dataBytes is a pointer to uint8_t[] where the first element is the MSB.
//!
//! \return Returns the signed-extend 32-bit result.
//
//*****************************************************************************
int32_t signExtend(const uint8_t dataBytes[])
{

    if (RES_24_BIT_ENABLED)
    {
        int32_t upperByte   = ((int32_t) dataBytes[0] << 24);
        int32_t middleByte  = ((int32_t) dataBytes[1] << 16);
        int32_t lowerByte   = ((int32_t) dataBytes[2] << 8);

        return (((int32_t) (upperByte | middleByte | lowerByte)) >> 8);     // Right-shift of signed data maintains signed bit
    }
    else
    {
        int32_t upperByte   = ((int32_t) dataBytes[0] << 24);
        int32_t lowerByte   = ((int32_t) dataBytes[1] << 16);

        return (((int32_t) (upperByte | lowerByte)) >> 16);                 // Right-shift of signed data maintains signed bit
    }

}


//****************************************************************************
//
// Internal functions
//
//****************************************************************************


//*****************************************************************************
//
//! Builds SPI TX data arrays according to number of opcodes provided and
//! currently programmed device word length.
//!
//! \fn uint8_t buildSPIarray(const uint16_t opcodeArray[], uint8_t numberOpcodes, uint8_t byteArray[])
//!
//! \param opcodeArray[] pointer to an array of 8-bit opcodes to use in the SPI command.
//! \param byteArray[] pointer to an array of 8-bit SPI bytes to send to the device.
//!
//! NOTE: The calling function must ensure it reserves sufficient memory for byteArray[]!
//!
//! \return number of bytes added to byteArray[].
//
//*****************************************************************************
uint8_t buildSPIarray(const uint8_t opcodeArray[], uint8_t byteArray[])
{

    const uint8_t numberOpcodes  = 2;  //Fixed at 2. 8 bits for the command + address, and 8 bits of data (W) or arbitrary bits (R)
    const uint8_t frontPadBytes  = (STATUS_ENABLED ? 1 : 0) + (RES_24_BIT_ENABLED ? 1 : 0);  //Determine where the opcode/data bytes will be placed in the array
    const uint8_t numberOfBytes  = numberOpcodes + frontPadBytes + (SPI_CRC_ENABLED ? 1 : 0);//Determine total number of bytes to clock

    //Place the command byte/data in the correct place in the array
    byteArray[frontPadBytes]     = opcodeArray[0];
    byteArray[frontPadBytes + 1] = opcodeArray[1];

    //Calculate/lookup CRC value
  #ifdef SPI_CRC_MODE
      byteArray[frontPadBytes + 2] = getCRC(opcodeArray, numberOpcodes, CRC_INITIAL_SEED);
  #endif


    return numberOfBytes;
}



//*****************************************************************************
//
//! Modifies CONFIG4 register data to maintain device operation according to
//! preselected mode(s) (DATA, SPI_CRC, STATUS).
//!
//! \fn uint8_t enforce_selected_device_mode(uint8_t data)
//!
//! \param data uint8_t register data.
//!
//! \return uint8_t modified register data.
//
//*****************************************************************************
uint8_t enforce_selected_device_modes(uint8_t data)
{

    ///////////////////////////////////////////////////////////////////////////
    // Enforce DATA Resolution setting

#ifdef RES_24_BIT_MODE
    // When writing to the CONFIG4 register, ensure DATA bit is NEVER set
    data &= CONFIG4_REG_DATA_0;
#else
    // When writing to the CONFIG4 register, ensure DATA bit is ALWAYS set
    data |= CONFIG4_REG_DATA_1;
#endif


    ///////////////////////////////////////////////////////////////////////////
    // Enforce SPI_CRC Setting

#ifdef SPI_CRC_MODE
    // When writing to the CONFIG4 register, ensure SPI_CRC bit is ALWAYS set
    data |= CONFIG4_REG_SPI_CRC_1;
#else
    // When writing to the CONFIG4 register, ensure SPI_CRC bit is NEVER set
    data &= CONFIG4_REG_SPI_CRC_0;
#endif


    ///////////////////////////////////////////////////////////////////////////
    // Enforce STATUS Byte Output Setting

#ifdef STATUS_BYTE_MODE
    // When writing to the CONFIG4 register, ensure STATUS bit is ALWAYS set
    data |= CONFIG4_REG_STATUS_1;
#else
    // When writing to the CONFIG4 register, ensure STATUS bit is NEVER set
    data &= CONFIG4_REG_STATUS_0;
#endif

    // Return modified register data
    return data;
}



//*****************************************************************************
//
//! Returns the ADS127L11 configured word length used for SPI communication.
//!
//! \fn uint8_t getWordByteLength(void)
//!
//! NOTE: It is important that the CONFIG4 register value stored in registerMap[]
//! remains in sync with the device. If these values get out of sync then SPI
//! communication may fail!
//!
//
//*****************************************************************************
uint8_t getWordByteLength(void)
{
    uint8_t frontPadBytes   = (STATUS_ENABLED ? 1 : 0) + (RES_24_BIT_ENABLED ? 1 : 0);
    uint8_t bytesPerWord    = frontPadBytes + 2 + (SPI_CRC_ENABLED ? 1 : 0);
    return bytesPerWord;
}
