/*
 * @file ads127l11.h
 *
 * @brief ADS127L11 Descriptor
 *
 * @copyright Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef ADS127L11_H_
#define ADS127L11_H_

// Standard libraries
#include <assert.h>
#include <stdint.h>
#include <stdbool.h>

// Custom libraries
#include "hal.h"
#include "crc.h"

//****************************************************************************
//
// Select the SPI mode and desired CONFIG4 register settings...
// NOTE: These settings will be enforced and not modifiable during runtime!
//
//****************************************************************************

/* Enable this to use 3 wire SPI mode*/
//#define SPI_3_WIRE

/* Enable this to use 24-bit resolution mode*/
#define RES_24_BIT_MODE

/* Enable this define statement to enable SPI_CRC */
//#define SPI_CRC_MODE

/* Enable this define statement to enable SPI STATUS bytes */
//#define STATUS_BYTE_MODE

/* Disable assertions when not in the CCS "Debug" configuration */
#ifndef _DEBUG
#define NDEBUG
#endif

//****************************************************************************
//
// ADC Channel data structure
//
//****************************************************************************

typedef struct
{
    uint8_t     status;
    uint8_t     crc;
    int32_t     adcdata;

} adc_channel_data;



//****************************************************************************
//
// Function prototypes
//
//****************************************************************************

void        adcStartup(void);
uint8_t     sendCommand(uint8_t opcode[]);
int32_t     readData(adc_channel_data *dataStruct);
uint8_t     readSingleRegister(uint8_t address);
void        writeSingleRegister(uint8_t address, uint8_t data);
void        resetDeviceByCommand(void);
void        resetDeviceByPattern(void);
void        restoreRegisterDefaults(void);

// Getter functions
uint16_t    getRegisterValue(uint8_t address);

// Helper functions
uint8_t     upperByte(uint16_t uint16_Word);
uint8_t     lowerByte(uint16_t uint16_Word);
uint16_t    combineBytes(uint8_t upperByte, uint8_t lowerByte);
int32_t     signExtend(const uint8_t dataBytes[]);



//****************************************************************************
//
// Register macros
//
//****************************************************************************


/** Returns true if Register Map CRC byte enable bit is set */
#define REGMAP_CRC_ENABLED      ((bool) (getRegisterValue(CONFIG4_REG_ADDRESS) & CONFIG4_REG_REG_CRC_1))

/** Returns true if SPI CRC byte enable bit is set */
#define SPI_CRC_ENABLED         ((bool) (getRegisterValue(CONFIG4_REG_ADDRESS) & CONFIG4_REG_SPI_CRC_1))

/** Returns true if STATUS byte enable bit is set */
#define STATUS_ENABLED          ((bool) (getRegisterValue(CONFIG4_REG_ADDRESS) & CONFIG4_REG_STATUS_1))

/** Returns true if 24 bit resolution is enabled */
#define RES_24_BIT_ENABLED      ((bool) !(getRegisterValue(CONFIG4_REG_ADDRESS) & CONFIG4_REG_DATA_0))

/** Returns the number of bytes in an SPI word */

#ifdef  SPI_CRC_MODE
#define SPI_BUFFER_SIZE         5 // maximum: STATUS byte pad + DATA byte pad + COMMAND byte 1 + COMMAND byte 2 + CRC byte
#else
#define SPI_BUFFER_SIZE         4 // maximum: STATUS byte pad + DATA byte pad + COMMAND byte 1 + COMMAND byte 2
#endif

//**********************************************************************************
//
// Device commands
//
//**********************************************************************************

#define OPCODE_NULL                             ((uint8_t) 0x00)
#define OPCODE_RREG                             ((uint8_t) 0x40)
#define OPCODE_WREG                             ((uint8_t) 0x80)

//**********************************************************************************
//
// Constants
//
//**********************************************************************************

#define NUM_REGISTERS                           ((uint8_t) 16)

//**********************************************************************************
//
// Register definitions
//
//**********************************************************************************


/* Register 0x00 (DEVID_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                           DEVID[7:0]                                                          |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* DEVID_REG register */
    #define DEVID_REG_ADDRESS												((uint8_t) 0x00)
    #define DEVID_REG_DEFAULT												((uint8_t) 0x00)



/* Register 0x01 (REVID_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                           REVID[7:0]                                                          |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* REVID_REG register */
    #define REVID_REG_ADDRESS												((uint8_t) 0x01)
    #define REVID_REG_DEFAULT												((uint8_t) 0x00)



/* Register 0x02 (STATUS_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |    CS_MODE    |    ALV_FLAG   |    POR_FLAG   |    SPI_ERR    |    REG_ERR    |     ADC_ERR   |   MOD_FLAG    |      DRDY     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* STATUS_REG register */
    #define STATUS_REG_ADDRESS												((uint8_t) 0x02)
    #define STATUS_REG_DEFAULT												((uint8_t) 0x00)

    /* CS Mode Field */
    #define STATUS_REG_CS_MODE_MASK    										((uint8_t) 0x80)
    #define STATUS_REG_CS_MODE_0											((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_CS_MODE_1											((uint8_t) 0x80)

    /* Analog Supply Low Voltage Field */
    #define STATUS_REG_ALV_FLAG_MASK										((uint8_t) 0x40)
    #define STATUS_REG_ALV_FLAG_0											((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_ALV_FLAG_1											((uint8_t) 0x40)

    /* POR Flag Field */
    #define STATUS_REG_POR_FLAG_MASK										((uint8_t) 0x20)
    #define STATUS_REG_POR_FLAG_0											((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_POR_FLAG_1											((uint8_t) 0x20)

    /* SPI CRC Error Field */
    #define STATUS_REG_SPI_ERR_MASK								    		((uint8_t) 0x10)
    #define STATUS_REG_SPI_ERR_0									    	((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_SPI_ERR_1									    	((uint8_t) 0x10)

    /* Register Map CRC Error Field */
    #define STATUS_REG_REG_ERR_MASK										    ((uint8_t) 0x08)
    #define STATUS_REG_REG_ERR_0											((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_REG_ERR_1											((uint8_t) 0x08)

    /* Internal ADC Error Field */
    #define STATUS_REG_ADC_ERR_MASK								    		((uint8_t) 0x04)
    #define STATUS_REG_ADC_ERR_0									    	((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_ADC_ERR_1									    	((uint8_t) 0x04)

    /* Modulator Overload Flag */
    #define STATUS_REG_MOD_FLAG_MASK								    	((uint8_t) 0x02)
    #define STATUS_REG_MOD_FLAG_0									    	((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_MOD_FLAG_1									    	((uint8_t) 0x02)

    /* Data Ready Bit */
    #define STATUS_REG_DRDY_MASK									    	((uint8_t) 0x01)
    #define STATUS_REG_DRDY_0										    	((uint8_t) 0x00)    // DEFAULT
    #define STATUS_REG_DRDY_1									    		((uint8_t) 0x01)



/* Register 0x03 (CONTROL_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                           RESET[5:0]                                          |     START     |      STOP     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* CONTROL_REG register */
    #define CONTROL_REG_ADDRESS                                            ((uint8_t) 0x03)
    #define CONTROL_REG_DEFAULT                                            ((uint8_t) 0x00)

    /* RESET field */
    #define CONTROL_REG_RESET_MASK                                         ((uint8_t) 0xFC)
    #define CONTROL_REG_RESET_0                                            ((uint8_t) 0x00)    // DEFAULT
    #define CONTROL_REG_RESET_1                                            ((uint8_t) 0x04)
    #define CONTROL_REG_RESET_COMMAND                                      ((uint8_t) 0x58)

    /* START field */
    #define CONTROL_REG_START_MASK                                         ((uint8_t) 0x02)
    #define CONTROL_REG_START_0                                            ((uint8_t) 0x00)    // DEFAULT
    #define CONTROL_REG_START_1                                            ((uint8_t) 0x02)

    /* STOP field */
    #define CONTROL_REG_STOP_MASK                                          ((uint8_t) 0x01)
    #define CONTROL_REG_STOP_0                                             ((uint8_t) 0x00)    // DEFAULT
    #define CONTROL_REG_STOP_1                                             ((uint8_t) 0x01)



/* Register 0x04 (MUX_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                           RESERVED                                            |             MUX[1:0]          |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* MUX_REG register */
    #define MUX_REG_ADDRESS									    			((uint8_t) 0x04)
    #define MUX_REG_DEFAULT									    			((uint8_t) 0x00)

    /* MUX field */
    #define MUX_REG_MUX_MASK								        		((uint8_t) 0x03)
    #define MUX_REG_MUX_NORMAL				       							((uint8_t) 0x00)    // DEFAULT
    #define MUX_REG_MUX_INVERTED                                            ((uint8_t) 0x01)
    #define MUX_REG_MUX_OFFSET_TEST                                         ((uint8_t) 0x02)
    #define MUX_REG_MUX_CM_TEST                                             ((uint8_t) 0x03)



/* Register 0x05 (CONFIG1_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |    RESERVED   |    REF_RNG    |    INP_RNG    |    VCM        |    REFP_BUF   |    RESERVED   |    AINP_BUF   |    AINN_BUFF  |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* CONFIG1 register */
    #define CONFIG1_REG_ADDRESS												((uint8_t) 0x05)
    #define CONFIG1_REG_DEFAULT												((uint8_t) 0x00)

    /* Voltage Reference Range field */
    #define CONFIG1_REG_REF_RNG_MASK										((uint8_t) 0x40)
    #define CONFIG1_REG_REF_RNG_0											((uint8_t) 0x00)    // DEFAULT
    #define CONFIG1_REG_REF_RNG_1											((uint8_t) 0x40)

    /* Input Range field */
    #define CONFIG1_REG_INP_RNG_MASK                                        ((uint8_t) 0x20)
    #define CONFIG1_REG_INP_RNG_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG1_REG_INP_RNG_1                                           ((uint8_t) 0x20)

    /* VCM Output field */
    #define CONFIG1_REG_VCM_MASK                                            ((uint8_t) 0x10)
    #define CONFIG1_REG_VCM_0                                               ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG1_REG_VCM_1                                               ((uint8_t) 0x10)

    /* Reference Positive Buffer field */
    #define CONFIG1_REG_REFP_BUFF_MASK                                      ((uint8_t) 0x08)
    #define CONFIG1_REG_REFP_BUFF_0                                         ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG1_REG_REFP_BUFF_1                                         ((uint8_t) 0x08)

    /* Analog Input Positive Buffer field */
    #define CONFIG1_REG_AINP_BUF_MASK                                       ((uint8_t) 0x02)
    #define CONFIG1_REG_AINP_BUF_0                                          ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG1_REG_AINP_BUF_1                                          ((uint8_t) 0x02)

    /* VCM Output field */
    #define CONFIG1_REG_AINN_BUF_MASK                                       ((uint8_t) 0x01)
    #define CONFIG1_REG_AINN_BUF_0                                          ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG1_REG_AINN_BUF_1                                          ((uint8_t) 0x01)


/* Register 0x06 (CONFIG2_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |    EXT_RNG    |    RESERVED   |    SDO_MODE   |        START_MODE[1:0]        |   SPEED_MODE  |    STBY_MODE  |     PWDN      |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* CONFIG2 register */
    #define CONFIG2_REG_ADDRESS                                             ((uint8_t) 0x06)
    #define CONFIG2_REG_DEFAULT                                             ((uint8_t) 0x00)

    /* Extended Input Range field */
    #define CONFIG2_REG_EXT_RNG_MASK                                        ((uint8_t) 0x80)
    #define CONFIG2_REG_EXT_RNG_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG2_REG_EXT_RNG_1                                           ((uint8_t) 0x80)

    /* SDO/nDRDY Mode field */
    #define CONFIG2_REG_SDO_MODE_MASK                                       ((uint8_t) 0x20)
    #define CONFIG2_REG_SDO_MODE_0                                          ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG2_REG_SDO_MODE_1                                          ((uint8_t) 0x20)

    /* START Mode field */
    #define CONFIG2_START_MODE_MASK                                         ((uint8_t) 0x18)
    #define CONFIG2_START_MODE_START_STOP                                   ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG2_START_MODE_ONESHOT                                      ((uint8_t) 0x08)
    #define CONFIG2_START_MODE_SYNC                                         ((uint8_t) 0x10)
    #define CONFIG2_START_MODE_RESERVED                                     ((uint8_t) 0x18)

    /* Speed Mode field */
    #define CONFIG2_REG_SPEED_MODE_MASK                                     ((uint8_t) 0x04)
    #define CONFIG2_REG_SPEED_MODE_0                                        ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG2_REG_SPEED_MODE_1                                        ((uint8_t) 0x04)

    /* Standby Mode field */
    #define CONFIG2_REG_STBY_MODE_MASK                                      ((uint8_t) 0x02)
    #define CONFIG2_REG_STBY_MODE_0                                         ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG2_REG_STBY_MODE_1                                         ((uint8_t) 0x02)

    /* Software Power Down field */
    #define CONFIG2_REG_PWDN_MASK                                           ((uint8_t) 0x01)
    #define CONFIG2_REG_PWDN_0                                              ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG2_REG_PWDN_1                                              ((uint8_t) 0x01)


/* Register 0x07 (CONFIG3_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                  DELAY[2:0]                   |                            FILTER[4:0]                                        |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* CONFIG3 register */
    #define CONFIG3_REG_ADDRESS										    	((uint8_t) 0x07)
    #define CONFIG3_REG_DEFAULT										    	((uint8_t) 0x00)

    /* Conversion-Start Delay Time field */
    #define CONFIG3_REG_DELAY_MASK											((uint8_t) 0xE0)
    #define CONFIG3_REG_DELAY_0	    										((uint8_t) 0x00)    // DEFAULT
    #define CONFIG3_REG_DELAY_4                                             ((uint8_t) 0x20)
    #define CONFIG3_REG_DELAY_8                                             ((uint8_t) 0x40)
    #define CONFIG3_REG_DELAY_16                                            ((uint8_t) 0x60)
    #define CONFIG3_REG_DELAY_32                                            ((uint8_t) 0x80)
    #define CONFIG3_REG_DELAY_128                                           ((uint8_t) 0xA0)
    #define CONFIG3_REG_DELAY_512                                           ((uint8_t) 0xC0)
    #define CONFIG3_REG_DELAY_1024                                          ((uint8_t) 0xE0)


    /* Filter field */
    #define CONFIG3_FILTER_MASK								    			((uint8_t) 0x1F)



/* Register 0x08 (CONFIG4_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |    CLK_SEL    |    CLK_DIV    |    OUT_DRV    |    RESERVED   |     DATA      |    SPI_CRC    |     REG_CRC   |     STATUS    |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* CONFIG4 register */
    #define CONFIG4_REG_ADDRESS                                             ((uint8_t) 0x08)
    #define CONFIG4_REG_DEFAULT                                             ((uint8_t) 0x00)

    /* Clock Selection field */
    #define CONFIG4_REG_CLK_SEL_MASK                                        ((uint8_t) 0x80)
    #define CONFIG4_REG_CLK_SEL_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_CLK_SEL_1                                           ((uint8_t) 0x80)

    /* External Clock Division Selection field */
    #define CONFIG4_REG_CLK_DIV_MASK                                        ((uint8_t) 0x40)
    #define CONFIG4_REG_CLK_DIV_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_CLK_DIV_1                                           ((uint8_t) 0x40)

    /* Digital OUtput Drive Selection Field */
    #define CONFIG4_REG_OUT_DRV_MASK                                        ((uint8_t) 0x20)
    #define CONFIG4_REG_OUT_DRV_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_OUT_DRV_1                                           ((uint8_t) 0x20)

    /* Data Resolution Selection field */
    #define CONFIG4_REG_DATA_MASK                                           ((uint8_t) 0x08)
    #define CONFIG4_REG_DATA_0                                              ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_DATA_1                                              ((uint8_t) 0x08)

    /* SPI CRC Enable field */
    #define CONFIG4_REG_SPI_CRC_MASK                                        ((uint8_t) 0x04)
    #define CONFIG4_REG_SPI_CRC_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_SPI_CRC_1                                           ((uint8_t) 0x04)

    /* Register Map CRC Enable field */
    #define CONFIG4_REG_REG_CRC_MASK                                        ((uint8_t) 0x02)
    #define CONFIG4_REG_REG_CRC_0                                           ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_REG_CRC_1                                           ((uint8_t) 0x02)

    /* Status Byte Output Enable field */
    #define CONFIG4_REG_STATUS_MASK                                         ((uint8_t) 0x01)
    #define CONFIG4_REG_STATUS_0                                            ((uint8_t) 0x00)    // DEFAULT
    #define CONFIG4_REG_STATUS_1                                            ((uint8_t) 0x01)


/* Register 0x09 (OFFSET_MSB_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       OFFSET[23:16]                                                           |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* OFFSET_MSB_REG register */
    #define OFFSET_MSB_REG_ADDRESS											((uint8_t) 0x09)
    #define OFFSET_MSB_REG_DEFAULT											((uint8_t) 0x00)


/* Register 0x0A (OFFSET_MID_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       OFFSET[15:8]                                                            |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* OFFSET_MID_REG register */
    #define OFFSET_MID_REG_ADDRESS                                          ((uint8_t) 0x0A)
    #define OFFSET_MID_REG_DEFAULT                                          ((uint8_t) 0x00)


/* Register 0x0B (OFFSET_LSB_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       OFFSET[7:0]                                                             |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* OFFSET_LSB_REG register */
    #define OFFSET_LSB_REG_ADDRESS                                          ((uint8_t) 0x0B)
    #define OFFSET_LSB_REG_DEFAULT                                          ((uint8_t) 0x00)


/* Register 0x0C (GAIN_MSB_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       GAIN[23:16]                                                             |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* GAIN_MSB_REG register */
    #define GAIN_MSB_REG_ADDRESS                                            ((uint8_t) 0x0C)
    #define GAIN_MSB_REG_DEFAULT                                            ((uint8_t) 0x40)


/* Register 0x0D (GAIN_MID_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       GAIN[15:8]                                                              |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* GAIN_MID_REG register */
    #define GAIN_MID_REG_ADDRESS                                            ((uint8_t) 0x0D)
    #define GAIN_MID_REG_DEFAULT                                            ((uint8_t) 0x00)


/* Register 0x0E (GAIN_LSB_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       GAIN[7:0]                                                               |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* GAIN_MSB_REG register */
    #define GAIN_LSB_REG_ADDRESS                                            ((uint8_t) 0x0E)
    #define GAIN_LSB_REG_DEFAULT                                            ((uint8_t) 0x00)


/* Register 0x0F (CRC_REG) definition
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |     Bit 7     |     Bit 6     |     Bit 5     |     Bit 4     |     Bit 3     |     Bit 2     |     Bit 1     |     Bit 0     |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 * |                                                       CRC[7:0]                                                                |
 * |-------------------------------------------------------------------------------------------------------------------------------|
 */

    /* CRC_REG register */
    #define CRC_REG_ADDRESS                                                 ((uint8_t) 0x0C)
    #define CRC_REG_DEFAULT                                                 ((uint8_t) 0x00)

#endif /* ADS127L11_H_ */
